# this file is @generated

from .common import BaseModel
from .endpoint_created_event_data import EndpointCreatedEventData


class EndpointCreatedEvent(BaseModel):
    """Sent when an endpoint is created."""

    data: EndpointCreatedEventData

    type: str



# this file is @generated
from .adobe_sign_config import AdobeSignConfig
from .adobe_sign_config_out import AdobeSignConfigOut
from .aggregate_event_types_out import AggregateEventTypesOut
from .api_token_censored_out import ApiTokenCensoredOut
from .api_token_expire_in import ApiTokenExpireIn
from .api_token_in import ApiTokenIn
from .api_token_out import ApiTokenOut
from .app_portal_access_in import AppPortalAccessIn
from .app_portal_access_out import AppPortalAccessOut
from .app_usage_stats_in import AppUsageStatsIn
from .app_usage_stats_out import AppUsageStatsOut
from .application_in import ApplicationIn
from .application_out import ApplicationOut
from .application_patch import ApplicationPatch
from .application_token_expire_in import ApplicationTokenExpireIn
from .background_task_data import BackgroundTaskData
from .background_task_finished_event import BackgroundTaskFinishedEvent
from .background_task_finished_event2 import BackgroundTaskFinishedEvent2
from .background_task_out import BackgroundTaskOut
from .background_task_status import BackgroundTaskStatus
from .background_task_type import BackgroundTaskType
from .connector_in import ConnectorIn
from .connector_kind import ConnectorKind
from .connector_out import ConnectorOut
from .cron_config import CronConfig
from .dashboard_access_out import DashboardAccessOut
from .docusign_config import DocusignConfig
from .docusign_config_out import DocusignConfigOut
from .endpoint_created_event import EndpointCreatedEvent
from .endpoint_created_event_data import EndpointCreatedEventData
from .endpoint_deleted_event import EndpointDeletedEvent
from .endpoint_deleted_event_data import EndpointDeletedEventData
from .endpoint_disabled_event import EndpointDisabledEvent
from .endpoint_disabled_event_data import EndpointDisabledEventData
from .endpoint_disabled_trigger import EndpointDisabledTrigger
from .endpoint_enabled_event import EndpointEnabledEvent
from .endpoint_enabled_event_data import EndpointEnabledEventData
from .endpoint_headers_in import EndpointHeadersIn
from .endpoint_headers_out import EndpointHeadersOut
from .endpoint_headers_patch_in import EndpointHeadersPatchIn
from .endpoint_in import EndpointIn
from .endpoint_message_out import EndpointMessageOut
from .endpoint_out import EndpointOut
from .endpoint_patch import EndpointPatch
from .endpoint_secret_out import EndpointSecretOut
from .endpoint_secret_rotate_in import EndpointSecretRotateIn
from .endpoint_stats import EndpointStats
from .endpoint_transformation_in import EndpointTransformationIn
from .endpoint_transformation_out import EndpointTransformationOut
from .endpoint_update import EndpointUpdate
from .endpoint_updated_event import EndpointUpdatedEvent
from .endpoint_updated_event_data import EndpointUpdatedEventData
from .environment_in import EnvironmentIn
from .environment_out import EnvironmentOut
from .event_example_in import EventExampleIn
from .event_type_from_open_api import EventTypeFromOpenApi
from .event_type_import_open_api_in import EventTypeImportOpenApiIn
from .event_type_import_open_api_out import EventTypeImportOpenApiOut
from .event_type_import_open_api_out_data import EventTypeImportOpenApiOutData
from .event_type_in import EventTypeIn
from .event_type_out import EventTypeOut
from .event_type_patch import EventTypePatch
from .event_type_update import EventTypeUpdate
from .expunge_all_contents_out import ExpungeAllContentsOut
from .github_config import GithubConfig
from .github_config_out import GithubConfigOut
from .hubspot_config import HubspotConfig
from .hubspot_config_out import HubspotConfigOut
from .ingest_endpoint_headers_in import IngestEndpointHeadersIn
from .ingest_endpoint_headers_out import IngestEndpointHeadersOut
from .ingest_endpoint_in import IngestEndpointIn
from .ingest_endpoint_out import IngestEndpointOut
from .ingest_endpoint_secret_in import IngestEndpointSecretIn
from .ingest_endpoint_secret_out import IngestEndpointSecretOut
from .ingest_endpoint_update import IngestEndpointUpdate
from .ingest_source_consumer_portal_access_in import IngestSourceConsumerPortalAccessIn
from .ingest_source_in import IngestSourceIn
from .ingest_source_out import IngestSourceOut
from .integration_in import IntegrationIn
from .integration_key_out import IntegrationKeyOut
from .integration_out import IntegrationOut
from .integration_update import IntegrationUpdate
from .list_response_api_token_censored_out import ListResponseApiTokenCensoredOut
from .list_response_application_out import ListResponseApplicationOut
from .list_response_background_task_out import ListResponseBackgroundTaskOut
from .list_response_endpoint_message_out import ListResponseEndpointMessageOut
from .list_response_endpoint_out import ListResponseEndpointOut
from .list_response_event_type_out import ListResponseEventTypeOut
from .list_response_ingest_endpoint_out import ListResponseIngestEndpointOut
from .list_response_ingest_source_out import ListResponseIngestSourceOut
from .list_response_integration_out import ListResponseIntegrationOut
from .list_response_message_attempt_out import ListResponseMessageAttemptOut
from .list_response_message_endpoint_out import ListResponseMessageEndpointOut
from .list_response_message_out import ListResponseMessageOut
from .list_response_operational_webhook_endpoint_out import (
    ListResponseOperationalWebhookEndpointOut,
)
from .message_attempt_exhausted_event import MessageAttemptExhaustedEvent
from .message_attempt_exhausted_event_data import MessageAttemptExhaustedEventData
from .message_attempt_failed_data import MessageAttemptFailedData
from .message_attempt_failing_event import MessageAttemptFailingEvent
from .message_attempt_failing_event_data import MessageAttemptFailingEventData
from .message_attempt_out import MessageAttemptOut
from .message_attempt_recovered_event import MessageAttemptRecoveredEvent
from .message_attempt_recovered_event_data import MessageAttemptRecoveredEventData
from .message_attempt_trigger_type import MessageAttemptTriggerType
from .message_endpoint_out import MessageEndpointOut
from .message_in import MessageIn
from .message_out import MessageOut
from .message_status import MessageStatus
from .operational_webhook_endpoint_headers_in import OperationalWebhookEndpointHeadersIn
from .operational_webhook_endpoint_headers_out import (
    OperationalWebhookEndpointHeadersOut,
)
from .operational_webhook_endpoint_in import OperationalWebhookEndpointIn
from .operational_webhook_endpoint_out import OperationalWebhookEndpointOut
from .operational_webhook_endpoint_secret_in import OperationalWebhookEndpointSecretIn
from .operational_webhook_endpoint_secret_out import OperationalWebhookEndpointSecretOut
from .operational_webhook_endpoint_update import OperationalWebhookEndpointUpdate
from .ordering import Ordering
from .polling_endpoint_consumer_seek_in import PollingEndpointConsumerSeekIn
from .polling_endpoint_consumer_seek_out import PollingEndpointConsumerSeekOut
from .polling_endpoint_message_out import PollingEndpointMessageOut
from .polling_endpoint_out import PollingEndpointOut
from .recover_in import RecoverIn
from .recover_out import RecoverOut
from .replay_in import ReplayIn
from .replay_out import ReplayOut
from .rotate_token_out import RotateTokenOut
from .segment_config import SegmentConfig
from .segment_config_out import SegmentConfigOut
from .shopify_config import ShopifyConfig
from .shopify_config_out import ShopifyConfigOut
from .slack_config import SlackConfig
from .slack_config_out import SlackConfigOut
from .status_code_class import StatusCodeClass
from .stripe_config import StripeConfig
from .stripe_config_out import StripeConfigOut
from .svix_config import SvixConfig
from .svix_config_out import SvixConfigOut
from .zoom_config import ZoomConfig
from .zoom_config_out import ZoomConfigOut

__all__ = [
    "BackgroundTaskData",
    "AdobeSignConfig",
    "AdobeSignConfigOut",
    "AggregateEventTypesOut",
    "ApiTokenCensoredOut",
    "ApiTokenExpireIn",
    "ApiTokenIn",
    "ApiTokenOut",
    "AppPortalAccessIn",
    "AppPortalAccessOut",
    "AppUsageStatsIn",
    "AppUsageStatsOut",
    "ApplicationIn",
    "ApplicationOut",
    "ApplicationPatch",
    "ApplicationTokenExpireIn",
    "BackgroundTaskFinishedEvent",
    "BackgroundTaskFinishedEvent2",
    "BackgroundTaskOut",
    "BackgroundTaskStatus",
    "BackgroundTaskType",
    "ConnectorIn",
    "ConnectorKind",
    "ConnectorOut",
    "CronConfig",
    "DashboardAccessOut",
    "DocusignConfig",
    "DocusignConfigOut",
    "EndpointCreatedEvent",
    "EndpointCreatedEventData",
    "EndpointDeletedEvent",
    "EndpointDeletedEventData",
    "EndpointDisabledEvent",
    "EndpointDisabledEventData",
    "EndpointDisabledTrigger",
    "EndpointEnabledEvent",
    "EndpointEnabledEventData",
    "EndpointHeadersIn",
    "EndpointHeadersOut",
    "EndpointHeadersPatchIn",
    "EndpointIn",
    "EndpointMessageOut",
    "EndpointOut",
    "EndpointPatch",
    "EndpointSecretOut",
    "EndpointSecretRotateIn",
    "EndpointStats",
    "EndpointTransformationIn",
    "EndpointTransformationOut",
    "EndpointUpdate",
    "EndpointUpdatedEvent",
    "EndpointUpdatedEventData",
    "EnvironmentIn",
    "EnvironmentOut",
    "EventExampleIn",
    "EventTypeFromOpenApi",
    "EventTypeImportOpenApiIn",
    "EventTypeImportOpenApiOut",
    "EventTypeImportOpenApiOutData",
    "EventTypeIn",
    "EventTypeOut",
    "EventTypePatch",
    "EventTypeUpdate",
    "ExpungeAllContentsOut",
    "GithubConfig",
    "GithubConfigOut",
    "HubspotConfig",
    "HubspotConfigOut",
    "IngestEndpointHeadersIn",
    "IngestEndpointHeadersOut",
    "IngestEndpointIn",
    "IngestEndpointOut",
    "IngestEndpointSecretIn",
    "IngestEndpointSecretOut",
    "IngestEndpointUpdate",
    "IngestSourceConsumerPortalAccessIn",
    "IngestSourceIn",
    "IngestSourceOut",
    "IntegrationIn",
    "IntegrationKeyOut",
    "IntegrationOut",
    "IntegrationUpdate",
    "ListResponseApiTokenCensoredOut",
    "ListResponseApplicationOut",
    "ListResponseBackgroundTaskOut",
    "ListResponseEndpointMessageOut",
    "ListResponseEndpointOut",
    "ListResponseEventTypeOut",
    "ListResponseIngestEndpointOut",
    "ListResponseIngestSourceOut",
    "ListResponseIntegrationOut",
    "ListResponseMessageAttemptOut",
    "ListResponseMessageEndpointOut",
    "ListResponseMessageOut",
    "ListResponseOperationalWebhookEndpointOut",
    "MessageAttemptExhaustedEvent",
    "MessageAttemptExhaustedEventData",
    "MessageAttemptFailedData",
    "MessageAttemptFailingEvent",
    "MessageAttemptFailingEventData",
    "MessageAttemptOut",
    "MessageAttemptRecoveredEvent",
    "MessageAttemptRecoveredEventData",
    "MessageAttemptTriggerType",
    "MessageEndpointOut",
    "MessageIn",
    "MessageOut",
    "MessageStatus",
    "OperationalWebhookEndpointHeadersIn",
    "OperationalWebhookEndpointHeadersOut",
    "OperationalWebhookEndpointIn",
    "OperationalWebhookEndpointOut",
    "OperationalWebhookEndpointSecretIn",
    "OperationalWebhookEndpointSecretOut",
    "OperationalWebhookEndpointUpdate",
    "Ordering",
    "PollingEndpointConsumerSeekIn",
    "PollingEndpointConsumerSeekOut",
    "PollingEndpointMessageOut",
    "PollingEndpointOut",
    "RecoverIn",
    "RecoverOut",
    "ReplayIn",
    "ReplayOut",
    "RotateTokenOut",
    "SegmentConfig",
    "SegmentConfigOut",
    "ShopifyConfig",
    "ShopifyConfigOut",
    "SlackConfig",
    "SlackConfigOut",
    "StatusCodeClass",
    "StripeConfig",
    "StripeConfigOut",
    "SvixConfig",
    "SvixConfigOut",
    "ZoomConfig",
    "ZoomConfigOut",
]



import requests

def test_openapi_json_is_available(svix_server_url: str) -> None:
    r = requests.get(f"{svix_server_url}/api/v1/openapi.json")
    assert r.status_code == 200
    assert r.headers["content-type"] == 'application/json'



# this file is @generated

from .common import BaseModel
from .endpoint_enabled_event_data import EndpointEnabledEventData


class EndpointEnabledEvent(BaseModel):
    """Sent when an endpoint has been enabled."""

    data: EndpointEnabledEventData

    type: str



# this file is @generated
import typing as t

from .common import BaseModel


class EndpointDeletedEventData(BaseModel):
    """Sent when an endpoint is created, updated, or deleted"""

    app_id: str
    """The Application's ID."""

    app_uid: t.Optional[str] = None
    """The Application's UID."""

    endpoint_id: str
    """The Endpoint's ID."""

    endpoint_uid: t.Optional[str] = None
    """The Endpoint's UID."""



# this file is @generated
import typing as t

from .common import BaseModel


class EndpointHeadersOut(BaseModel):
    """The value of the headers is returned in the `headers` field.

    Sensitive headers that have been redacted are returned in the sensitive field."""

    headers: t.Dict[str, str]

    sensitive: t.List[str]



# This file is @generated
import typing as t
from dataclasses import dataclass

from .. import models
from ..models import BackgroundTaskOut, ListResponseBackgroundTaskOut
from .common import ApiBase, BaseOptions, serialize_params


@dataclass
class BackgroundTaskListOptions(BaseOptions):
    status: t.Optional[models.BackgroundTaskStatus] = None
    """Filter the response based on the status."""
    task: t.Optional[models.BackgroundTaskType] = None
    """Filter the response based on the type."""
    limit: t.Optional[int] = None
    """Limit the number of returned items"""
    iterator: t.Optional[str] = None
    """The iterator returned from a prior invocation"""
    order: t.Optional[models.Ordering] = None
    """The sorting order of the returned items"""

    def _query_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "status": self.status,
                "task": self.task,
                "limit": self.limit,
                "iterator": self.iterator,
                "order": self.order,
            }
        )


class BackgroundTaskAsync(ApiBase):
    async def list(
        self, options: BackgroundTaskListOptions = BackgroundTaskListOptions()
    ) -> ListResponseBackgroundTaskOut:
        """List background tasks executed in the past 90 days."""
        response = await self._request_asyncio(
            method="get",
            path="/api/v1/background-task",
            path_params={},
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return ListResponseBackgroundTaskOut.model_validate(response.json())

    async def get(self, task_id: str) -> BackgroundTaskOut:
        """Get a background task by ID."""
        response = await self._request_asyncio(
            method="get",
            path="/api/v1/background-task/{task_id}",
            path_params={
                "task_id": task_id,
            },
        )
        return BackgroundTaskOut.model_validate(response.json())


class BackgroundTask(ApiBase):
    def list(
        self, options: BackgroundTaskListOptions = BackgroundTaskListOptions()
    ) -> ListResponseBackgroundTaskOut:
        """List background tasks executed in the past 90 days."""
        response = self._request_sync(
            method="get",
            path="/api/v1/background-task",
            path_params={},
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return ListResponseBackgroundTaskOut.model_validate(response.json())

    def get(self, task_id: str) -> BackgroundTaskOut:
        """Get a background task by ID."""
        response = self._request_sync(
            method="get",
            path="/api/v1/background-task/{task_id}",
            path_params={
                "task_id": task_id,
            },
        )
        return BackgroundTaskOut.model_validate(response.json())



# this file is @generated
import typing as t

from .common import BaseModel
from .polling_endpoint_message_out import PollingEndpointMessageOut


class PollingEndpointOut(BaseModel):
    data: t.List[PollingEndpointMessageOut]

    done: bool

    iterator: str



# this file is @generated
import typing as t
from datetime import datetime

from .common import BaseModel


class ApiTokenOut(BaseModel):
    created_at: datetime

    expires_at: t.Optional[datetime] = None

    id: str
    """The GlobalApplicationToken's ID."""

    name: t.Optional[str] = None

    scopes: t.Optional[t.List[str]] = None

    token: str



# This file is @generated
import typing as t
from dataclasses import dataclass
from datetime import datetime

from ..models import (
    ExpungeAllContentsOut,
    ListResponseMessageOut,
    MessageIn,
    MessageOut,
)
from .common import ApiBase, BaseOptions, serialize_params
from .message_poller import (
    MessagePoller,
    MessagePollerAsync,
)


@dataclass
class MessageListOptions(BaseOptions):
    limit: t.Optional[int] = None
    """Limit the number of returned items"""
    iterator: t.Optional[str] = None
    """The iterator returned from a prior invocation"""
    channel: t.Optional[str] = None
    """Filter response based on the channel."""
    before: t.Optional[datetime] = None
    """Only include items created before a certain date."""
    after: t.Optional[datetime] = None
    """Only include items created after a certain date."""
    with_content: t.Optional[bool] = None
    """When `true` message payloads are included in the response."""
    tag: t.Optional[str] = None
    """Filter messages matching the provided tag."""
    event_types: t.Optional[t.List[str]] = None
    """Filter response based on the event type"""

    def _query_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "limit": self.limit,
                "iterator": self.iterator,
                "channel": self.channel,
                "before": self.before,
                "after": self.after,
                "with_content": self.with_content,
                "tag": self.tag,
                "event_types": self.event_types,
            }
        )


@dataclass
class MessageCreateOptions(BaseOptions):
    with_content: t.Optional[bool] = None
    """When `true`, message payloads are included in the response."""
    idempotency_key: t.Optional[str] = None

    def _query_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "with_content": self.with_content,
            }
        )

    def _header_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "idempotency-key": self.idempotency_key,
            }
        )


@dataclass
class MessageExpungeAllContentsOptions(BaseOptions):
    idempotency_key: t.Optional[str] = None

    def _header_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "idempotency-key": self.idempotency_key,
            }
        )


@dataclass
class MessageGetOptions(BaseOptions):
    with_content: t.Optional[bool] = None
    """When `true` message payloads are included in the response."""

    def _query_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "with_content": self.with_content,
            }
        )


def message_in_raw(
    event_type: str, payload: str, content_type: t.Optional[str] = None
) -> MessageIn:
    """
    Creates a `MessageIn` with a raw string payload.

    The payload is not normalized on the server. Normally, payloads are required
    to be JSON, and Svix will minify the payload before sending the webhook
    (for example, by removing extraneous whitespace or unnecessarily escaped
    characters in strings). With this function, the payload will be sent
    "as is", without any minification or other processing.

    Args:
        event_type (str): The event type's name Example: `user.signup`.
        payload (str): Serialized message payload.
        content_type (str?): The value to use for the Content-Type header of the
            webhook sent by Svix, overwriting the default of `application/json`
            if specified.
    """
    transformations_params: t.Dict[str, t.Any] = {
        "rawPayload": payload,
    }
    if content_type is not None:
        transformations_params["headers"] = {"content-type": content_type}

    return MessageIn(
        event_type=event_type,
        payload={},
        transformations_params=transformations_params,
    )


class MessageAsync(ApiBase):
    @property
    def poller(self) -> MessagePollerAsync:
        return MessagePollerAsync(self._client)

    async def list(
        self, app_id: str, options: MessageListOptions = MessageListOptions()
    ) -> ListResponseMessageOut:
        """List all of the application's messages.

        The `before` and `after` parameters let you filter all items created before or after a certain date. These can be used alongside an iterator to paginate over results
        within a certain window.

        Note that by default this endpoint is limited to retrieving 90 days' worth of data
        relative to now or, if an iterator is provided, 90 days before/after the time indicated
        by the iterator ID. If you require data beyond those time ranges, you will need to explicitly
        set the `before` or `after` parameter as appropriate."""
        response = await self._request_asyncio(
            method="get",
            path="/api/v1/app/{app_id}/msg",
            path_params={
                "app_id": app_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return ListResponseMessageOut.model_validate(response.json())

    async def create(
        self,
        app_id: str,
        message_in: MessageIn,
        options: MessageCreateOptions = MessageCreateOptions(),
    ) -> MessageOut:
        """Creates a new message and dispatches it to all of the application's endpoints.

        The `eventId` is an optional custom unique ID. It's verified to be unique only up to a day, after that no verification will be made.
        If a message with the same `eventId` already exists for the application, a 409 conflict error will be returned.

        The `eventType` indicates the type and schema of the event. All messages of a certain `eventType` are expected to have the same schema. Endpoints can choose to only listen to specific event types.
        Messages can also have `channels`, which similar to event types let endpoints filter by them. Unlike event types, messages can have multiple channels, and channels don't imply a specific message content or schema.

        The `payload` property is the webhook's body (the actual webhook message). Svix supports payload sizes of up to 1MiB, though it's generally a good idea to keep webhook payloads small, probably no larger than 40kb."""
        response = await self._request_asyncio(
            method="post",
            path="/api/v1/app/{app_id}/msg",
            path_params={
                "app_id": app_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=message_in.model_dump_json(exclude_unset=True, by_alias=True),
        )
        return MessageOut.model_validate(response.json())

    async def expunge_all_contents(
        self,
        app_id: str,
        options: MessageExpungeAllContentsOptions = MessageExpungeAllContentsOptions(),
    ) -> ExpungeAllContentsOut:
        """Delete all message payloads for the application.

        This operation is only available in the <a href="https://svix.com/pricing" target="_blank">Enterprise</a> plan."""
        response = await self._request_asyncio(
            method="post",
            path="/api/v1/app/{app_id}/msg/expunge-all-contents",
            path_params={
                "app_id": app_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return ExpungeAllContentsOut.model_validate(response.json())

    async def get(
        self, app_id: str, msg_id: str, options: MessageGetOptions = MessageGetOptions()
    ) -> MessageOut:
        """Get a message by its ID or eventID."""
        response = await self._request_asyncio(
            method="get",
            path="/api/v1/app/{app_id}/msg/{msg_id}",
            path_params={
                "app_id": app_id,
                "msg_id": msg_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return MessageOut.model_validate(response.json())

    async def expunge_content(self, app_id: str, msg_id: str) -> None:
        """Delete the given message's payload.

        Useful in cases when a message was accidentally sent with sensitive content.
        The message can't be replayed or resent once its payload has been deleted or expired."""
        await self._request_asyncio(
            method="delete",
            path="/api/v1/app/{app_id}/msg/{msg_id}/content",
            path_params={
                "app_id": app_id,
                "msg_id": msg_id,
            },
        )


class Message(ApiBase):
    @property
    def poller(self) -> MessagePoller:
        return MessagePoller(self._client)

    def list(
        self, app_id: str, options: MessageListOptions = MessageListOptions()
    ) -> ListResponseMessageOut:
        """List all of the application's messages.

        The `before` and `after` parameters let you filter all items created before or after a certain date. These can be used alongside an iterator to paginate over results
        within a certain window.

        Note that by default this endpoint is limited to retrieving 90 days' worth of data
        relative to now or, if an iterator is provided, 90 days before/after the time indicated
        by the iterator ID. If you require data beyond those time ranges, you will need to explicitly
        set the `before` or `after` parameter as appropriate."""
        response = self._request_sync(
            method="get",
            path="/api/v1/app/{app_id}/msg",
            path_params={
                "app_id": app_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return ListResponseMessageOut.model_validate(response.json())

    def create(
        self,
        app_id: str,
        message_in: MessageIn,
        options: MessageCreateOptions = MessageCreateOptions(),
    ) -> MessageOut:
        """Creates a new message and dispatches it to all of the application's endpoints.

        The `eventId` is an optional custom unique ID. It's verified to be unique only up to a day, after that no verification will be made.
        If a message with the same `eventId` already exists for the application, a 409 conflict error will be returned.

        The `eventType` indicates the type and schema of the event. All messages of a certain `eventType` are expected to have the same schema. Endpoints can choose to only listen to specific event types.
        Messages can also have `channels`, which similar to event types let endpoints filter by them. Unlike event types, messages can have multiple channels, and channels don't imply a specific message content or schema.

        The `payload` property is the webhook's body (the actual webhook message). Svix supports payload sizes of up to 1MiB, though it's generally a good idea to keep webhook payloads small, probably no larger than 40kb."""
        response = self._request_sync(
            method="post",
            path="/api/v1/app/{app_id}/msg",
            path_params={
                "app_id": app_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=message_in.model_dump_json(exclude_unset=True, by_alias=True),
        )
        return MessageOut.model_validate(response.json())

    def expunge_all_contents(
        self,
        app_id: str,
        options: MessageExpungeAllContentsOptions = MessageExpungeAllContentsOptions(),
    ) -> ExpungeAllContentsOut:
        """Delete all message payloads for the application.

        This operation is only available in the <a href="https://svix.com/pricing" target="_blank">Enterprise</a> plan."""
        response = self._request_sync(
            method="post",
            path="/api/v1/app/{app_id}/msg/expunge-all-contents",
            path_params={
                "app_id": app_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return ExpungeAllContentsOut.model_validate(response.json())

    def get(
        self, app_id: str, msg_id: str, options: MessageGetOptions = MessageGetOptions()
    ) -> MessageOut:
        """Get a message by its ID or eventID."""
        response = self._request_sync(
            method="get",
            path="/api/v1/app/{app_id}/msg/{msg_id}",
            path_params={
                "app_id": app_id,
                "msg_id": msg_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return MessageOut.model_validate(response.json())

    def expunge_content(self, app_id: str, msg_id: str) -> None:
        """Delete the given message's payload.

        Useful in cases when a message was accidentally sent with sensitive content.
        The message can't be replayed or resent once its payload has been deleted or expired."""
        self._request_sync(
            method="delete",
            path="/api/v1/app/{app_id}/msg/{msg_id}/content",
            path_params={
                "app_id": app_id,
                "msg_id": msg_id,
            },
        )



# This file is @generated
import typing as t
from dataclasses import dataclass
from datetime import datetime

from ..models import (
    PollingEndpointConsumerSeekIn,
    PollingEndpointConsumerSeekOut,
    PollingEndpointOut,
)
from .common import ApiBase, BaseOptions, serialize_params


@dataclass
class MessagePollerPollOptions(BaseOptions):
    limit: t.Optional[int] = None
    """Limit the number of returned items"""
    iterator: t.Optional[str] = None
    """The iterator returned from a prior invocation"""
    event_type: t.Optional[str] = None
    """Filters messages sent with this event type (optional)."""
    channel: t.Optional[str] = None
    """Filters messages sent with this channel (optional)."""
    after: t.Optional[datetime] = None

    def _query_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "limit": self.limit,
                "iterator": self.iterator,
                "event_type": self.event_type,
                "channel": self.channel,
                "after": self.after,
            }
        )


@dataclass
class MessagePollerConsumerPollOptions(BaseOptions):
    limit: t.Optional[int] = None
    """Limit the number of returned items"""
    iterator: t.Optional[str] = None
    """The iterator returned from a prior invocation"""

    def _query_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "limit": self.limit,
                "iterator": self.iterator,
            }
        )


@dataclass
class MessagePollerConsumerSeekOptions(BaseOptions):
    idempotency_key: t.Optional[str] = None

    def _header_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "idempotency-key": self.idempotency_key,
            }
        )


class MessagePollerAsync(ApiBase):
    async def poll(
        self,
        app_id: str,
        sink_id: str,
        options: MessagePollerPollOptions = MessagePollerPollOptions(),
    ) -> PollingEndpointOut:
        """Reads the stream of created messages for an application, filtered on the Sink's event types and Channels."""
        response = await self._request_asyncio(
            method="get",
            path="/api/v1/app/{app_id}/poller/{sink_id}",
            path_params={
                "app_id": app_id,
                "sink_id": sink_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return PollingEndpointOut.model_validate(response.json())

    async def consumer_poll(
        self,
        app_id: str,
        sink_id: str,
        consumer_id: str,
        options: MessagePollerConsumerPollOptions = MessagePollerConsumerPollOptions(),
    ) -> PollingEndpointOut:
        """Reads the stream of created messages for an application, filtered on the Sink's event types and
        Channels, using server-managed iterator tracking."""
        response = await self._request_asyncio(
            method="get",
            path="/api/v1/app/{app_id}/poller/{sink_id}/consumer/{consumer_id}",
            path_params={
                "app_id": app_id,
                "sink_id": sink_id,
                "consumer_id": consumer_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return PollingEndpointOut.model_validate(response.json())

    async def consumer_seek(
        self,
        app_id: str,
        sink_id: str,
        consumer_id: str,
        polling_endpoint_consumer_seek_in: PollingEndpointConsumerSeekIn,
        options: MessagePollerConsumerSeekOptions = MessagePollerConsumerSeekOptions(),
    ) -> PollingEndpointConsumerSeekOut:
        """Sets the starting offset for the consumer of a polling endpoint."""
        response = await self._request_asyncio(
            method="post",
            path="/api/v1/app/{app_id}/poller/{sink_id}/consumer/{consumer_id}/seek",
            path_params={
                "app_id": app_id,
                "sink_id": sink_id,
                "consumer_id": consumer_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=polling_endpoint_consumer_seek_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return PollingEndpointConsumerSeekOut.model_validate(response.json())


class MessagePoller(ApiBase):
    def poll(
        self,
        app_id: str,
        sink_id: str,
        options: MessagePollerPollOptions = MessagePollerPollOptions(),
    ) -> PollingEndpointOut:
        """Reads the stream of created messages for an application, filtered on the Sink's event types and Channels."""
        response = self._request_sync(
            method="get",
            path="/api/v1/app/{app_id}/poller/{sink_id}",
            path_params={
                "app_id": app_id,
                "sink_id": sink_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return PollingEndpointOut.model_validate(response.json())

    def consumer_poll(
        self,
        app_id: str,
        sink_id: str,
        consumer_id: str,
        options: MessagePollerConsumerPollOptions = MessagePollerConsumerPollOptions(),
    ) -> PollingEndpointOut:
        """Reads the stream of created messages for an application, filtered on the Sink's event types and
        Channels, using server-managed iterator tracking."""
        response = self._request_sync(
            method="get",
            path="/api/v1/app/{app_id}/poller/{sink_id}/consumer/{consumer_id}",
            path_params={
                "app_id": app_id,
                "sink_id": sink_id,
                "consumer_id": consumer_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return PollingEndpointOut.model_validate(response.json())

    def consumer_seek(
        self,
        app_id: str,
        sink_id: str,
        consumer_id: str,
        polling_endpoint_consumer_seek_in: PollingEndpointConsumerSeekIn,
        options: MessagePollerConsumerSeekOptions = MessagePollerConsumerSeekOptions(),
    ) -> PollingEndpointConsumerSeekOut:
        """Sets the starting offset for the consumer of a polling endpoint."""
        response = self._request_sync(
            method="post",
            path="/api/v1/app/{app_id}/poller/{sink_id}/consumer/{consumer_id}/seek",
            path_params={
                "app_id": app_id,
                "sink_id": sink_id,
                "consumer_id": consumer_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=polling_endpoint_consumer_seek_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return PollingEndpointConsumerSeekOut.model_validate(response.json())



# this file is @generated

from .common import BaseModel
from .endpoint_disabled_event_data import EndpointDisabledEventData


class EndpointDisabledEvent(BaseModel):
    """Sent when an endpoint has been automatically disabled after continuous failures, or manually via an API call."""

    data: EndpointDisabledEventData

    type: str



# This file is @generated
from .common import ApiBase
from .management_authentication import (
    ManagementAuthentication,
    ManagementAuthenticationAsync,
)


class ManagementAsync(ApiBase):
    @property
    def authentication(self) -> ManagementAuthenticationAsync:
        return ManagementAuthenticationAsync(self._client)


class Management(ApiBase):
    @property
    def authentication(self) -> ManagementAuthentication:
        return ManagementAuthentication(self._client)



# This file is @generated
import typing as t
from dataclasses import dataclass

from ..models import DashboardAccessOut, IngestSourceConsumerPortalAccessIn
from .common import ApiBase, BaseOptions, serialize_params
from .ingest_endpoint import (
    IngestEndpoint,
    IngestEndpointAsync,
)
from .ingest_source import (
    IngestSource,
    IngestSourceAsync,
)


@dataclass
class IngestDashboardOptions(BaseOptions):
    idempotency_key: t.Optional[str] = None

    def _header_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "idempotency-key": self.idempotency_key,
            }
        )


class IngestAsync(ApiBase):
    @property
    def endpoint(self) -> IngestEndpointAsync:
        return IngestEndpointAsync(self._client)

    @property
    def source(self) -> IngestSourceAsync:
        return IngestSourceAsync(self._client)

    async def dashboard(
        self,
        source_id: str,
        ingest_source_consumer_portal_access_in: IngestSourceConsumerPortalAccessIn,
        options: IngestDashboardOptions = IngestDashboardOptions(),
    ) -> DashboardAccessOut:
        """Get access to the Ingest Source Consumer Portal."""
        response = await self._request_asyncio(
            method="post",
            path="/ingest/api/v1/source/{source_id}/dashboard",
            path_params={
                "source_id": source_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=ingest_source_consumer_portal_access_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return DashboardAccessOut.model_validate(response.json())


class Ingest(ApiBase):
    @property
    def endpoint(self) -> IngestEndpoint:
        return IngestEndpoint(self._client)

    @property
    def source(self) -> IngestSource:
        return IngestSource(self._client)

    def dashboard(
        self,
        source_id: str,
        ingest_source_consumer_portal_access_in: IngestSourceConsumerPortalAccessIn,
        options: IngestDashboardOptions = IngestDashboardOptions(),
    ) -> DashboardAccessOut:
        """Get access to the Ingest Source Consumer Portal."""
        response = self._request_sync(
            method="post",
            path="/ingest/api/v1/source/{source_id}/dashboard",
            path_params={
                "source_id": source_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=ingest_source_consumer_portal_access_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return DashboardAccessOut.model_validate(response.json())



# this file is @generated
import typing as t
from datetime import datetime

from pydantic import ModelWrapValidatorHandler, model_validator
from typing_extensions import Self

from .adobe_sign_config_out import AdobeSignConfigOut
from .common import BaseModel
from .cron_config import CronConfig
from .docusign_config_out import DocusignConfigOut
from .github_config_out import GithubConfigOut
from .hubspot_config_out import HubspotConfigOut
from .segment_config_out import SegmentConfigOut
from .shopify_config_out import ShopifyConfigOut
from .slack_config_out import SlackConfigOut
from .stripe_config_out import StripeConfigOut
from .svix_config_out import SvixConfigOut
from .zoom_config_out import ZoomConfigOut


class IngestSourceOut(BaseModel):
    created_at: datetime

    id: str
    """The Source's ID."""

    ingest_url: t.Optional[str] = None

    name: str

    uid: t.Optional[str] = None
    """The Source's UID."""

    updated_at: datetime

    type: t.Union[
        t.Literal["generic-webhook"],
        t.Literal["cron"],
        t.Literal["adobe-sign"],
        t.Literal["beehiiv"],
        t.Literal["brex"],
        t.Literal["clerk"],
        t.Literal["docusign"],
        t.Literal["github"],
        t.Literal["guesty"],
        t.Literal["hubspot"],
        t.Literal["incident-io"],
        t.Literal["lithic"],
        t.Literal["nash"],
        t.Literal["pleo"],
        t.Literal["replicate"],
        t.Literal["resend"],
        t.Literal["safebase"],
        t.Literal["sardine"],
        t.Literal["segment"],
        t.Literal["shopify"],
        t.Literal["slack"],
        t.Literal["stripe"],
        t.Literal["stych"],
        t.Literal["svix"],
        t.Literal["zoom"],
    ]
    config: t.Union[
        t.Dict[str, t.Any],
        CronConfig,
        AdobeSignConfigOut,
        SvixConfigOut,
        DocusignConfigOut,
        GithubConfigOut,
        HubspotConfigOut,
        SegmentConfigOut,
        ShopifyConfigOut,
        SlackConfigOut,
        StripeConfigOut,
        ZoomConfigOut,
    ]

    @model_validator(mode="wrap")
    @classmethod
    def validate_model(
        cls, data: t.Any, handler: ModelWrapValidatorHandler[Self]
    ) -> Self:
        output = handler(data)
        if output.type == "generic-webhook":
            output.config = data.get("config", {})
        elif output.type == "cron":
            output.config = CronConfig.model_validate(data.get("config", {}))
        elif output.type == "adobe-sign":
            output.config = AdobeSignConfigOut.model_validate(data.get("config", {}))
        elif output.type == "beehiiv":
            output.config = SvixConfigOut.model_validate(data.get("config", {}))
        elif output.type == "brex":
            output.config = SvixConfigOut.model_validate(data.get("config", {}))
        elif output.type == "clerk":
            output.config = SvixConfigOut.model_validate(data.get("config", {}))
        elif output.type == "docusign":
            output.config = DocusignConfigOut.model_validate(data.get("config", {}))
        elif output.type == "github":
            output.config = GithubConfigOut.model_validate(data.get("config", {}))
        elif output.type == "guesty":
            output.config = SvixConfigOut.model_validate(data.get("config", {}))
        elif output.type == "hubspot":
            output.config = HubspotConfigOut.model_validate(data.get("config", {}))
        elif output.type == "incident-io":
            output.config = SvixConfigOut.model_validate(data.get("config", {}))
        elif output.type == "lithic":
            output.config = SvixConfigOut.model_validate(data.get("config", {}))
        elif output.type == "nash":
            output.config = SvixConfigOut.model_validate(data.get("config", {}))
        elif output.type == "pleo":
            output.config = SvixConfigOut.model_validate(data.get("config", {}))
        elif output.type == "replicate":
            output.config = SvixConfigOut.model_validate(data.get("config", {}))
        elif output.type == "resend":
            output.config = SvixConfigOut.model_validate(data.get("config", {}))
        elif output.type == "safebase":
            output.config = SvixConfigOut.model_validate(data.get("config", {}))
        elif output.type == "sardine":
            output.config = SvixConfigOut.model_validate(data.get("config", {}))
        elif output.type == "segment":
            output.config = SegmentConfigOut.model_validate(data.get("config", {}))
        elif output.type == "shopify":
            output.config = ShopifyConfigOut.model_validate(data.get("config", {}))
        elif output.type == "slack":
            output.config = SlackConfigOut.model_validate(data.get("config", {}))
        elif output.type == "stripe":
            output.config = StripeConfigOut.model_validate(data.get("config", {}))
        elif output.type == "stych":
            output.config = SvixConfigOut.model_validate(data.get("config", {}))
        elif output.type == "svix":
            output.config = SvixConfigOut.model_validate(data.get("config", {}))
        elif output.type == "zoom":
            output.config = ZoomConfigOut.model_validate(data.get("config", {}))
        else:
            raise ValueError(f"Unexpected type `{output.type}`")
        return output



# this file is @generated

from .background_task_finished_event2 import BackgroundTaskFinishedEvent2
from .common import BaseModel


class BackgroundTaskFinishedEvent(BaseModel):
    """Sent when a background task is finished."""

    data: BackgroundTaskFinishedEvent2

    type: str



# this file is @generated
import typing as t

from .common import BaseModel


class CronConfig(BaseModel):
    content_type: t.Optional[str] = None
    """Override the default content-type.

    Recommended if the payload is not JSON."""

    payload: str

    schedule: str



# This file is @generated
import typing as t
from dataclasses import dataclass

from .. import models
from ..models import (
    ApiTokenExpireIn,
    ApiTokenIn,
    ApiTokenOut,
    ListResponseApiTokenCensoredOut,
)
from .common import ApiBase, BaseOptions, serialize_params


@dataclass
class ManagementAuthenticationListApiTokensOptions(BaseOptions):
    limit: t.Optional[int] = None
    """Limit the number of returned items"""
    iterator: t.Optional[str] = None
    """The iterator returned from a prior invocation"""
    order: t.Optional[models.Ordering] = None
    """The sorting order of the returned items"""

    def _query_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "limit": self.limit,
                "iterator": self.iterator,
                "order": self.order,
            }
        )


@dataclass
class ManagementAuthenticationCreateApiTokenOptions(BaseOptions):
    idempotency_key: t.Optional[str] = None

    def _header_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "idempotency-key": self.idempotency_key,
            }
        )


@dataclass
class ManagementAuthenticationExpireApiTokenOptions(BaseOptions):
    idempotency_key: t.Optional[str] = None

    def _header_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "idempotency-key": self.idempotency_key,
            }
        )


class ManagementAuthenticationAsync(ApiBase):
    async def list_api_tokens(
        self,
        options: ManagementAuthenticationListApiTokensOptions = ManagementAuthenticationListApiTokensOptions(),
    ) -> ListResponseApiTokenCensoredOut:
        """List all API Tokens."""
        response = await self._request_asyncio(
            method="get",
            path="/api/v1/management/authentication/api-token",
            path_params={},
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return ListResponseApiTokenCensoredOut.model_validate(response.json())

    async def create_api_token(
        self,
        api_token_in: ApiTokenIn,
        options: ManagementAuthenticationCreateApiTokenOptions = ManagementAuthenticationCreateApiTokenOptions(),
    ) -> ApiTokenOut:
        """Create a new API Token."""
        response = await self._request_asyncio(
            method="post",
            path="/api/v1/management/authentication/api-token",
            path_params={},
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=api_token_in.model_dump_json(exclude_unset=True, by_alias=True),
        )
        return ApiTokenOut.model_validate(response.json())

    async def expire_api_token(
        self,
        key_id: str,
        api_token_expire_in: ApiTokenExpireIn,
        options: ManagementAuthenticationExpireApiTokenOptions = ManagementAuthenticationExpireApiTokenOptions(),
    ) -> None:
        """Expire the selected API Token."""
        await self._request_asyncio(
            method="post",
            path="/api/v1/management/authentication/api-token/{key_id}/expire",
            path_params={
                "key_id": key_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=api_token_expire_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )


class ManagementAuthentication(ApiBase):
    def list_api_tokens(
        self,
        options: ManagementAuthenticationListApiTokensOptions = ManagementAuthenticationListApiTokensOptions(),
    ) -> ListResponseApiTokenCensoredOut:
        """List all API Tokens."""
        response = self._request_sync(
            method="get",
            path="/api/v1/management/authentication/api-token",
            path_params={},
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return ListResponseApiTokenCensoredOut.model_validate(response.json())

    def create_api_token(
        self,
        api_token_in: ApiTokenIn,
        options: ManagementAuthenticationCreateApiTokenOptions = ManagementAuthenticationCreateApiTokenOptions(),
    ) -> ApiTokenOut:
        """Create a new API Token."""
        response = self._request_sync(
            method="post",
            path="/api/v1/management/authentication/api-token",
            path_params={},
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=api_token_in.model_dump_json(exclude_unset=True, by_alias=True),
        )
        return ApiTokenOut.model_validate(response.json())

    def expire_api_token(
        self,
        key_id: str,
        api_token_expire_in: ApiTokenExpireIn,
        options: ManagementAuthenticationExpireApiTokenOptions = ManagementAuthenticationExpireApiTokenOptions(),
    ) -> None:
        """Expire the selected API Token."""
        self._request_sync(
            method="post",
            path="/api/v1/management/authentication/api-token/{key_id}/expire",
            path_params={
                "key_id": key_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=api_token_expire_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )



# this file is @generated

from .common import BaseModel


class OperationalWebhookEndpointSecretOut(BaseModel):
    key: str
    """The endpoint's verification secret.

    Format: `base64` encoded random bytes optionally prefixed with `whsec_`.
    It is recommended to not set this and let the server generate the secret."""



# this file is @generated

from .common import BaseModel
from .message_attempt_recovered_event_data import MessageAttemptRecoveredEventData


class MessageAttemptRecoveredEvent(BaseModel):
    """Sent on a successful dispatch after an earlier failure op webhook has already been sent."""

    data: MessageAttemptRecoveredEventData

    type: str



# This file is @generated
import typing as t
from dataclasses import dataclass

from .. import models
from ..models import (
    IngestSourceIn,
    IngestSourceOut,
    ListResponseIngestSourceOut,
    RotateTokenOut,
)
from .common import ApiBase, BaseOptions, serialize_params


@dataclass
class IngestSourceListOptions(BaseOptions):
    limit: t.Optional[int] = None
    """Limit the number of returned items"""
    iterator: t.Optional[str] = None
    """The iterator returned from a prior invocation"""
    order: t.Optional[models.Ordering] = None
    """The sorting order of the returned items"""

    def _query_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "limit": self.limit,
                "iterator": self.iterator,
                "order": self.order,
            }
        )


@dataclass
class IngestSourceCreateOptions(BaseOptions):
    idempotency_key: t.Optional[str] = None

    def _header_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "idempotency-key": self.idempotency_key,
            }
        )


@dataclass
class IngestSourceRotateTokenOptions(BaseOptions):
    idempotency_key: t.Optional[str] = None

    def _header_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "idempotency-key": self.idempotency_key,
            }
        )


class IngestSourceAsync(ApiBase):
    async def list(
        self, options: IngestSourceListOptions = IngestSourceListOptions()
    ) -> ListResponseIngestSourceOut:
        """List of all the organization's Ingest Sources."""
        response = await self._request_asyncio(
            method="get",
            path="/ingest/api/v1/source",
            path_params={},
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return ListResponseIngestSourceOut.model_validate(response.json())

    async def create(
        self,
        ingest_source_in: IngestSourceIn,
        options: IngestSourceCreateOptions = IngestSourceCreateOptions(),
    ) -> IngestSourceOut:
        """Create Ingest Source."""
        response = await self._request_asyncio(
            method="post",
            path="/ingest/api/v1/source",
            path_params={},
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=ingest_source_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return IngestSourceOut.model_validate(response.json())

    async def get(self, source_id: str) -> IngestSourceOut:
        """Get an Ingest Source by id or uid."""
        response = await self._request_asyncio(
            method="get",
            path="/ingest/api/v1/source/{source_id}",
            path_params={
                "source_id": source_id,
            },
        )
        return IngestSourceOut.model_validate(response.json())

    async def update(
        self, source_id: str, ingest_source_in: IngestSourceIn
    ) -> IngestSourceOut:
        """Update an Ingest Source."""
        response = await self._request_asyncio(
            method="put",
            path="/ingest/api/v1/source/{source_id}",
            path_params={
                "source_id": source_id,
            },
            json_body=ingest_source_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return IngestSourceOut.model_validate(response.json())

    async def delete(self, source_id: str) -> None:
        """Delete an Ingest Source."""
        await self._request_asyncio(
            method="delete",
            path="/ingest/api/v1/source/{source_id}",
            path_params={
                "source_id": source_id,
            },
        )

    async def rotate_token(
        self,
        source_id: str,
        options: IngestSourceRotateTokenOptions = IngestSourceRotateTokenOptions(),
    ) -> RotateTokenOut:
        """Rotate the Ingest Source's Url Token.

        This will rotate the ingest source's token, which is used to
        construct the unique `ingestUrl` for the source. Previous tokens
        will remain valid for 48 hours after rotation. The token can be
        rotated a maximum of three times within the 48-hour period."""
        response = await self._request_asyncio(
            method="post",
            path="/ingest/api/v1/source/{source_id}/token/rotate",
            path_params={
                "source_id": source_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return RotateTokenOut.model_validate(response.json())


class IngestSource(ApiBase):
    def list(
        self, options: IngestSourceListOptions = IngestSourceListOptions()
    ) -> ListResponseIngestSourceOut:
        """List of all the organization's Ingest Sources."""
        response = self._request_sync(
            method="get",
            path="/ingest/api/v1/source",
            path_params={},
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return ListResponseIngestSourceOut.model_validate(response.json())

    def create(
        self,
        ingest_source_in: IngestSourceIn,
        options: IngestSourceCreateOptions = IngestSourceCreateOptions(),
    ) -> IngestSourceOut:
        """Create Ingest Source."""
        response = self._request_sync(
            method="post",
            path="/ingest/api/v1/source",
            path_params={},
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=ingest_source_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return IngestSourceOut.model_validate(response.json())

    def get(self, source_id: str) -> IngestSourceOut:
        """Get an Ingest Source by id or uid."""
        response = self._request_sync(
            method="get",
            path="/ingest/api/v1/source/{source_id}",
            path_params={
                "source_id": source_id,
            },
        )
        return IngestSourceOut.model_validate(response.json())

    def update(
        self, source_id: str, ingest_source_in: IngestSourceIn
    ) -> IngestSourceOut:
        """Update an Ingest Source."""
        response = self._request_sync(
            method="put",
            path="/ingest/api/v1/source/{source_id}",
            path_params={
                "source_id": source_id,
            },
            json_body=ingest_source_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return IngestSourceOut.model_validate(response.json())

    def delete(self, source_id: str) -> None:
        """Delete an Ingest Source."""
        self._request_sync(
            method="delete",
            path="/ingest/api/v1/source/{source_id}",
            path_params={
                "source_id": source_id,
            },
        )

    def rotate_token(
        self,
        source_id: str,
        options: IngestSourceRotateTokenOptions = IngestSourceRotateTokenOptions(),
    ) -> RotateTokenOut:
        """Rotate the Ingest Source's Url Token.

        This will rotate the ingest source's token, which is used to
        construct the unique `ingestUrl` for the source. Previous tokens
        will remain valid for 48 hours after rotation. The token can be
        rotated a maximum of three times within the 48-hour period."""
        response = self._request_sync(
            method="post",
            path="/ingest/api/v1/source/{source_id}/token/rotate",
            path_params={
                "source_id": source_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return RotateTokenOut.model_validate(response.json())



# this file is @generated
from enum import IntEnum


class StatusCodeClass(IntEnum):
    """The different classes of HTTP status codes:
    - CodeNone = 0
    - Code1xx = 100
    - Code2xx = 200
    - Code3xx = 300
    - Code4xx = 400
    - Code5xx = 500"""

    CODE_NONE = 0
    CODE1XX = 100
    CODE2XX = 200
    CODE3XX = 300
    CODE4XX = 400
    CODE5XX = 500

    def __str__(self) -> str:
        return str(self.value)



# this file is @generated
import typing as t

from .application_in import ApplicationIn
from .common import BaseModel


class MessageIn(BaseModel):
    application: t.Optional[ApplicationIn] = None
    """Optionally creates a new application alongside the message.

    If the application id or uid that is used in the path already exists, this argument is ignored."""

    channels: t.Optional[t.List[str]] = None
    """List of free-form identifiers that endpoints can filter by"""

    event_id: t.Optional[str] = None
    """Optional unique identifier for the message"""

    event_type: str
    """The event type's name"""

    payload: t.Dict[str, t.Any]
    """JSON payload to send as the request body of the webhook.

    We also support sending non-JSON payloads. Please contact us for more information."""

    payload_retention_hours: t.Optional[int] = None
    """Optional number of hours to retain the message payload. Note that this is mutually exclusive with `payloadRetentionPeriod`."""

    payload_retention_period: t.Optional[int] = None
    """Optional number of days to retain the message payload. Defaults to 90. Note that this is mutually exclusive with `payloadRetentionHours`."""

    tags: t.Optional[t.List[str]] = None
    """List of free-form tags that can be filtered by when listing messages"""

    transformations_params: t.Optional[t.Dict[str, t.Any]] = None
    """Extra parameters to pass to Transformations (for future use)"""



# This file is @generated
from .common import ApiBase
from .operational_webhook_endpoint import (
    OperationalWebhookEndpoint,
    OperationalWebhookEndpointAsync,
)


class OperationalWebhookAsync(ApiBase):
    @property
    def endpoint(self) -> OperationalWebhookEndpointAsync:
        return OperationalWebhookEndpointAsync(self._client)


class OperationalWebhook(ApiBase):
    @property
    def endpoint(self) -> OperationalWebhookEndpoint:
        return OperationalWebhookEndpoint(self._client)



# This file is @generated
import typing as t
from dataclasses import dataclass

from deprecated import deprecated

from ..models import (
    ApplicationTokenExpireIn,
    AppPortalAccessIn,
    AppPortalAccessOut,
    DashboardAccessOut,
)
from .common import ApiBase, BaseOptions, serialize_params


@dataclass
class AuthenticationAppPortalAccessOptions(BaseOptions):
    idempotency_key: t.Optional[str] = None

    def _header_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "idempotency-key": self.idempotency_key,
            }
        )


@dataclass
class AuthenticationExpireAllOptions(BaseOptions):
    idempotency_key: t.Optional[str] = None

    def _header_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "idempotency-key": self.idempotency_key,
            }
        )


@dataclass
class AuthenticationLogoutOptions(BaseOptions):
    idempotency_key: t.Optional[str] = None

    def _header_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "idempotency-key": self.idempotency_key,
            }
        )


@dataclass
class AuthenticationDashboardAccessOptions(BaseOptions):
    idempotency_key: t.Optional[str] = None

    def _header_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "idempotency-key": self.idempotency_key,
            }
        )


class AuthenticationAsync(ApiBase):
    async def app_portal_access(
        self,
        app_id: str,
        app_portal_access_in: AppPortalAccessIn,
        options: AuthenticationAppPortalAccessOptions = AuthenticationAppPortalAccessOptions(),
    ) -> AppPortalAccessOut:
        """Use this function to get magic links (and authentication codes) for connecting your users to the Consumer Application Portal."""
        response = await self._request_asyncio(
            method="post",
            path="/api/v1/auth/app-portal-access/{app_id}",
            path_params={
                "app_id": app_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=app_portal_access_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return AppPortalAccessOut.model_validate(response.json())

    async def expire_all(
        self,
        app_id: str,
        application_token_expire_in: ApplicationTokenExpireIn,
        options: AuthenticationExpireAllOptions = AuthenticationExpireAllOptions(),
    ) -> None:
        """Expire all of the tokens associated with a specific application."""
        await self._request_asyncio(
            method="post",
            path="/api/v1/auth/app/{app_id}/expire-all",
            path_params={
                "app_id": app_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=application_token_expire_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )

    @deprecated
    async def dashboard_access(
        self,
        app_id: str,
        options: AuthenticationDashboardAccessOptions = AuthenticationDashboardAccessOptions(),
    ) -> DashboardAccessOut:
        """Deprecated: Please use `app_portal_access` instead."""
        response = await self._request_asyncio(
            method="post",
            path="/api/v1/auth/dashboard-access/{app_id}",
            path_params={
                "app_id": app_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return DashboardAccessOut.model_validate(response.json())

    async def logout(
        self, options: AuthenticationLogoutOptions = AuthenticationLogoutOptions()
    ) -> None:
        """Logout an app token.

        Trying to log out other tokens will fail."""
        await self._request_asyncio(
            method="post",
            path="/api/v1/auth/logout",
            path_params={},
            query_params=options._query_params(),
            header_params=options._header_params(),
        )


class Authentication(ApiBase):
    def app_portal_access(
        self,
        app_id: str,
        app_portal_access_in: AppPortalAccessIn,
        options: AuthenticationAppPortalAccessOptions = AuthenticationAppPortalAccessOptions(),
    ) -> AppPortalAccessOut:
        """Use this function to get magic links (and authentication codes) for connecting your users to the Consumer Application Portal."""
        response = self._request_sync(
            method="post",
            path="/api/v1/auth/app-portal-access/{app_id}",
            path_params={
                "app_id": app_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=app_portal_access_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return AppPortalAccessOut.model_validate(response.json())

    def expire_all(
        self,
        app_id: str,
        application_token_expire_in: ApplicationTokenExpireIn,
        options: AuthenticationExpireAllOptions = AuthenticationExpireAllOptions(),
    ) -> None:
        """Expire all of the tokens associated with a specific application."""
        self._request_sync(
            method="post",
            path="/api/v1/auth/app/{app_id}/expire-all",
            path_params={
                "app_id": app_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=application_token_expire_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )

    @deprecated
    def dashboard_access(
        self,
        app_id: str,
        options: AuthenticationDashboardAccessOptions = AuthenticationDashboardAccessOptions(),
    ) -> DashboardAccessOut:
        """Deprecated: Please use `app_portal_access` instead."""
        response = self._request_sync(
            method="post",
            path="/api/v1/auth/dashboard-access/{app_id}",
            path_params={
                "app_id": app_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return DashboardAccessOut.model_validate(response.json())

    def logout(
        self, options: AuthenticationLogoutOptions = AuthenticationLogoutOptions()
    ) -> None:
        """Logout an app token.

        Trying to log out other tokens will fail."""
        self._request_sync(
            method="post",
            path="/api/v1/auth/logout",
            path_params={},
            query_params=options._query_params(),
            header_params=options._header_params(),
        )



import svix
from svix.api import MessageListOptions,IngestSourceIn, CronConfig
from svix.api.ingest_source import IngestSource

import httpretty

@httpretty.activate(verbose=True, allow_net_connect=False)
def test_octothorpe_in_query_param():
    svx = svix.Svix("token",svix.SvixOptions(server_url="http://test.example"))
    httpretty.register_uri(
        httpretty.GET,
        "http://test.example/api/v1/app/app_id/msg?tag=test%23test",
        body='{"data":[],"iterator":null,"prevIterator":null,"done":true}'
    )
    svx.message.list("app_id",MessageListOptions(tag="test#test"))

@httpretty.activate(verbose=True, allow_net_connect=False)
def test_struct_enum_with_fields():
    svx = svix.Svix("token", svix.SvixOptions(server_url="http://test.example"))
    ingest_source = IngestSource(svx._client)
    httpretty.register_uri(
        httpretty.POST,
        "http://test.example/ingest/api/v1/source",
        body='{"type":"cron","config":{"content_type":"mendy/tired","payload":"@hello there","schedule":"* * * * *"},"id":"src_2yZwUhtgs5Ai8T9yRQJXA","uid":"unique-identifier","name":"string","ingestUrl":"http://example.com","createdAt":"2019-08-24T14:15:22Z","updatedAt":"2019-08-24T14:15:22Z"}',
    )
    source_in = IngestSourceIn(
        name="name",
        type="cron",
        config=CronConfig(content_type="asd", payload="asd", schedule="asd"),
    )
    res = ingest_source.create(source_in)
    assert isinstance(res.config, CronConfig)

    assert res.config.content_type == "mendy/tired"
    assert res.config.payload == "@hello there"
    assert res.config.schedule == "* * * * *"


@httpretty.activate(verbose=True, allow_net_connect=False)
def test_struct_enum_without_fields():
    svx = svix.Svix("token", svix.SvixOptions(server_url="http://test.example"))
    ingest_source = IngestSource(svx._client)
    httpretty.register_uri(
        httpretty.POST,
        "http://test.example/ingest/api/v1/source",
        body='{"type":"generic-webhook","config":{},"id":"src_2yZwUhtgs5Ai8T9yRQJXA","uid":"unique-identifier","name":"string","ingestUrl":"http://example.com","createdAt":"2019-08-24T14:15:22Z","updatedAt":"2019-08-24T14:15:22Z"}',
    )
    source_in = IngestSourceIn(name="name", type="generic-webhook", config={})

    res = ingest_source.create(source_in)

    assert isinstance(res.config, dict)
    assert res.config == {}



# this file is @generated
import typing as t

from .common import BaseModel


class IngestSourceConsumerPortalAccessIn(BaseModel):
    expiry: t.Optional[int] = None
    """How long the token will be valid for, in seconds.

    Valid values are between 1 hour and 7 days. The default is 7 days."""

    read_only: t.Optional[bool] = None
    """Whether the app portal should be in read-only mode."""



# this file is @generated
import typing as t

from pydantic import Field

from .common import BaseModel
from .event_type_from_open_api import EventTypeFromOpenApi


class EventTypeImportOpenApiOutData(BaseModel):
    modified: t.List[str]

    to_modify: t.Optional[t.List[EventTypeFromOpenApi]] = Field(
        default=None, alias="to_modify"
    )



# this file is @generated
import typing as t

from .common import BaseModel


class ApplicationPatch(BaseModel):
    metadata: t.Optional[t.Dict[str, str]] = None

    name: t.Optional[str] = None

    rate_limit: t.Optional[int] = None

    uid: t.Optional[str] = None
    """The Application's UID."""



# this file is @generated

from .common import BaseModel
from .endpoint_updated_event_data import EndpointUpdatedEventData


class EndpointUpdatedEvent(BaseModel):
    """Sent when an endpoint is updated."""

    data: EndpointUpdatedEventData

    type: str



# this file is @generated
import typing as t
from datetime import datetime

from .common import BaseModel
from .endpoint_disabled_trigger import EndpointDisabledTrigger


class EndpointDisabledEventData(BaseModel):
    """Sent when an endpoint has been automatically disabled after continuous failures, or manually via an API call."""

    app_id: str
    """The Application's ID."""

    app_uid: t.Optional[str] = None
    """The Application's UID."""

    endpoint_id: str
    """The Endpoint's ID."""

    endpoint_uid: t.Optional[str] = None
    """The Endpoint's UID."""

    fail_since: t.Optional[datetime] = None

    trigger: t.Optional[EndpointDisabledTrigger] = None



import pydantic
from pydantic.alias_generators import to_camel


class BaseModel(pydantic.BaseModel):
    model_config = pydantic.ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True,
        json_encoders={set: lambda v: list(v)},
    )



# this file is @generated
from datetime import datetime

from .common import BaseModel


class MessageAttemptFailedData(BaseModel):
    id: str
    """The MessageAttempt's ID."""

    response_status_code: int

    timestamp: datetime



from ..models import (
    AdobeSignConfig,
    AdobeSignConfigOut,
    AggregateEventTypesOut,
    ApiTokenCensoredOut,
    ApiTokenExpireIn,
    ApiTokenIn,
    ApiTokenOut,
    ApplicationIn,
    ApplicationOut,
    ApplicationPatch,
    ApplicationTokenExpireIn,
    AppPortalAccessIn,
    AppPortalAccessOut,
    AppUsageStatsIn,
    AppUsageStatsOut,
    BackgroundTaskData,
    BackgroundTaskFinishedEvent,
    BackgroundTaskFinishedEvent2,
    BackgroundTaskOut,
    BackgroundTaskStatus,
    BackgroundTaskType,
    ConnectorIn,
    ConnectorKind,
    ConnectorOut,
    CronConfig,
    DashboardAccessOut,
    DocusignConfig,
    DocusignConfigOut,
    EndpointCreatedEvent,
    EndpointCreatedEventData,
    EndpointDeletedEvent,
    EndpointDeletedEventData,
    EndpointDisabledEvent,
    EndpointDisabledEventData,
    EndpointDisabledTrigger,
    EndpointEnabledEvent,
    EndpointEnabledEventData,
    EndpointHeadersIn,
    EndpointHeadersOut,
    EndpointHeadersPatchIn,
    EndpointIn,
    EndpointMessageOut,
    EndpointOut,
    EndpointPatch,
    EndpointSecretOut,
    EndpointSecretRotateIn,
    EndpointStats,
    EndpointTransformationIn,
    EndpointTransformationOut,
    EndpointUpdate,
    EndpointUpdatedEvent,
    EndpointUpdatedEventData,
    EnvironmentIn,
    EnvironmentOut,
    EventExampleIn,
    EventTypeFromOpenApi,
    EventTypeImportOpenApiIn,
    EventTypeImportOpenApiOut,
    EventTypeImportOpenApiOutData,
    EventTypeIn,
    EventTypeOut,
    EventTypePatch,
    EventTypeUpdate,
    ExpungeAllContentsOut,
    GithubConfig,
    GithubConfigOut,
    HubspotConfig,
    HubspotConfigOut,
    IngestEndpointHeadersIn,
    IngestEndpointHeadersOut,
    IngestEndpointIn,
    IngestEndpointOut,
    IngestEndpointSecretIn,
    IngestEndpointSecretOut,
    IngestEndpointUpdate,
    IngestSourceConsumerPortalAccessIn,
    IngestSourceIn,
    IngestSourceOut,
    IntegrationIn,
    IntegrationKeyOut,
    IntegrationOut,
    IntegrationUpdate,
    ListResponseApiTokenCensoredOut,
    ListResponseApplicationOut,
    ListResponseBackgroundTaskOut,
    ListResponseEndpointMessageOut,
    ListResponseEndpointOut,
    ListResponseEventTypeOut,
    ListResponseIngestEndpointOut,
    ListResponseIngestSourceOut,
    ListResponseIntegrationOut,
    ListResponseMessageAttemptOut,
    ListResponseMessageEndpointOut,
    ListResponseMessageOut,
    ListResponseOperationalWebhookEndpointOut,
    MessageAttemptExhaustedEvent,
    MessageAttemptExhaustedEventData,
    MessageAttemptFailedData,
    MessageAttemptFailingEvent,
    MessageAttemptFailingEventData,
    MessageAttemptOut,
    MessageAttemptRecoveredEvent,
    MessageAttemptRecoveredEventData,
    MessageAttemptTriggerType,
    MessageEndpointOut,
    MessageIn,
    MessageOut,
    MessageStatus,
    OperationalWebhookEndpointHeadersIn,
    OperationalWebhookEndpointHeadersOut,
    OperationalWebhookEndpointIn,
    OperationalWebhookEndpointOut,
    OperationalWebhookEndpointSecretIn,
    OperationalWebhookEndpointSecretOut,
    OperationalWebhookEndpointUpdate,
    Ordering,
    PollingEndpointConsumerSeekIn,
    PollingEndpointConsumerSeekOut,
    PollingEndpointMessageOut,
    PollingEndpointOut,
    RecoverIn,
    RecoverOut,
    ReplayIn,
    ReplayOut,
    RotateTokenOut,
    SegmentConfig,
    SegmentConfigOut,
    ShopifyConfig,
    ShopifyConfigOut,
    SlackConfig,
    SlackConfigOut,
    StatusCodeClass,
    StripeConfig,
    StripeConfigOut,
    SvixConfig,
    SvixConfigOut,
    ZoomConfig,
    ZoomConfigOut,
)
from .application import (
    Application,
    ApplicationAsync,
    ApplicationCreateOptions,
    ApplicationListOptions,
)
from .authentication import (
    Authentication,
    AuthenticationAppPortalAccessOptions,
    AuthenticationAsync,
    AuthenticationDashboardAccessOptions,
    AuthenticationExpireAllOptions,
    AuthenticationLogoutOptions,
)
from .background_task import (
    BackgroundTask,
    BackgroundTaskAsync,
    BackgroundTaskListOptions,
)
from .endpoint import (
    Endpoint,
    EndpointAsync,
    EndpointCreateOptions,
    EndpointGetStatsOptions,
    EndpointListOptions,
    EndpointRecoverOptions,
    EndpointReplayMissingOptions,
    EndpointRotateSecretOptions,
    EndpointSendExampleOptions,
)
from .event_type import (
    EventType,
    EventTypeAsync,
    EventTypeCreateOptions,
    EventTypeDeleteOptions,
    EventTypeImportOpenapiOptions,
    EventTypeListOptions,
)
from .integration import (
    Integration,
    IntegrationAsync,
    IntegrationCreateOptions,
    IntegrationListOptions,
    IntegrationRotateKeyOptions,
)
from .management_authentication import (
    ManagementAuthentication,
    ManagementAuthenticationAsync,
    ManagementAuthenticationCreateApiTokenOptions,
    ManagementAuthenticationExpireApiTokenOptions,
    ManagementAuthenticationListApiTokensOptions,
)
from .message import (
    Message,
    MessageAsync,
    MessageCreateOptions,
    MessageExpungeAllContentsOptions,
    MessageGetOptions,
    MessageListOptions,
)
from .message_attempt import (
    MessageAttempt,
    MessageAttemptAsync,
    MessageAttemptListAttemptedDestinationsOptions,
    MessageAttemptListAttemptedMessagesOptions,
    MessageAttemptListByEndpointOptions,
    MessageAttemptListByMsgOptions,
    MessageAttemptResendOptions,
)
from .message_poller import (
    MessagePoller,
    MessagePollerAsync,
    MessagePollerConsumerPollOptions,
    MessagePollerConsumerSeekOptions,
    MessagePollerPollOptions,
)
from .operational_webhook_endpoint import (
    OperationalWebhookEndpoint,
    OperationalWebhookEndpointAsync,
    OperationalWebhookEndpointCreateOptions,
    OperationalWebhookEndpointListOptions,
    OperationalWebhookEndpointRotateSecretOptions,
)
from .statistics import Statistics, StatisticsAggregateAppStatsOptions, StatisticsAsync
from .svix import DEFAULT_SERVER_URL, Svix, SvixAsync, SvixOptions

__all__ = [
    "Svix",
    "SvixAsync",
    "SvixOptions",
    "DEFAULT_SERVER_URL",
    "Application",
    "ApplicationAsync",
    "ApplicationListOptions",
    "ApplicationCreateOptions",
    "Authentication",
    "AuthenticationAsync",
    "AuthenticationAppPortalAccessOptions",
    "AuthenticationExpireAllOptions",
    "AuthenticationLogoutOptions",
    "AuthenticationDashboardAccessOptions",
    "BackgroundTask",
    "BackgroundTaskAsync",
    "BackgroundTaskListOptions",
    "Endpoint",
    "EndpointAsync",
    "EndpointListOptions",
    "EndpointCreateOptions",
    "EndpointRecoverOptions",
    "EndpointReplayMissingOptions",
    "EndpointRotateSecretOptions",
    "EndpointSendExampleOptions",
    "EndpointGetStatsOptions",
    "EventType",
    "EventTypeAsync",
    "EventTypeListOptions",
    "EventTypeCreateOptions",
    "EventTypeImportOpenapiOptions",
    "EventTypeDeleteOptions",
    "Integration",
    "IntegrationAsync",
    "IntegrationListOptions",
    "IntegrationCreateOptions",
    "IntegrationRotateKeyOptions",
    "ManagementAuthentication",
    "ManagementAuthenticationAsync",
    "ManagementAuthenticationListApiTokensOptions",
    "ManagementAuthenticationCreateApiTokenOptions",
    "ManagementAuthenticationExpireApiTokenOptions",
    "Message",
    "MessageAsync",
    "MessageListOptions",
    "MessageCreateOptions",
    "MessageExpungeAllContentsOptions",
    "MessageGetOptions",
    "MessagePoller",
    "MessagePollerAsync",
    "MessagePollerPollOptions",
    "MessagePollerConsumerPollOptions",
    "MessagePollerConsumerSeekOptions",
    "MessageAttempt",
    "MessageAttemptAsync",
    "MessageAttemptListByEndpointOptions",
    "MessageAttemptListByMsgOptions",
    "MessageAttemptListAttemptedMessagesOptions",
    "MessageAttemptListAttemptedDestinationsOptions",
    "MessageAttemptResendOptions",
    "OperationalWebhookEndpoint",
    "OperationalWebhookEndpointAsync",
    "OperationalWebhookEndpointListOptions",
    "OperationalWebhookEndpointCreateOptions",
    "OperationalWebhookEndpointRotateSecretOptions",
    "Statistics",
    "StatisticsAsync",
    "StatisticsAggregateAppStatsOptions",
    "BackgroundTaskData",
    "AdobeSignConfig",
    "AdobeSignConfigOut",
    "AggregateEventTypesOut",
    "ApiTokenCensoredOut",
    "ApiTokenExpireIn",
    "ApiTokenIn",
    "ApiTokenOut",
    "AppPortalAccessIn",
    "AppPortalAccessOut",
    "AppUsageStatsIn",
    "AppUsageStatsOut",
    "ApplicationIn",
    "ApplicationOut",
    "ApplicationPatch",
    "ApplicationTokenExpireIn",
    "BackgroundTaskFinishedEvent",
    "BackgroundTaskFinishedEvent2",
    "BackgroundTaskOut",
    "BackgroundTaskStatus",
    "BackgroundTaskType",
    "ConnectorIn",
    "ConnectorKind",
    "ConnectorOut",
    "CronConfig",
    "DashboardAccessOut",
    "DocusignConfig",
    "DocusignConfigOut",
    "EndpointCreatedEvent",
    "EndpointCreatedEventData",
    "EndpointDeletedEvent",
    "EndpointDeletedEventData",
    "EndpointDisabledEvent",
    "EndpointDisabledEventData",
    "EndpointDisabledTrigger",
    "EndpointEnabledEvent",
    "EndpointEnabledEventData",
    "EndpointHeadersIn",
    "EndpointHeadersOut",
    "EndpointHeadersPatchIn",
    "EndpointIn",
    "EndpointMessageOut",
    "EndpointOut",
    "EndpointPatch",
    "EndpointSecretOut",
    "EndpointSecretRotateIn",
    "EndpointStats",
    "EndpointTransformationIn",
    "EndpointTransformationOut",
    "EndpointUpdate",
    "EndpointUpdatedEvent",
    "EndpointUpdatedEventData",
    "EnvironmentIn",
    "EnvironmentOut",
    "EventExampleIn",
    "EventTypeFromOpenApi",
    "EventTypeImportOpenApiIn",
    "EventTypeImportOpenApiOut",
    "EventTypeImportOpenApiOutData",
    "EventTypeIn",
    "EventTypeOut",
    "EventTypePatch",
    "EventTypeUpdate",
    "ExpungeAllContentsOut",
    "GithubConfig",
    "GithubConfigOut",
    "HubspotConfig",
    "HubspotConfigOut",
    "IngestEndpointHeadersIn",
    "IngestEndpointHeadersOut",
    "IngestEndpointIn",
    "IngestEndpointOut",
    "IngestEndpointSecretIn",
    "IngestEndpointSecretOut",
    "IngestEndpointUpdate",
    "IngestSourceConsumerPortalAccessIn",
    "IngestSourceIn",
    "IngestSourceOut",
    "IntegrationIn",
    "IntegrationKeyOut",
    "IntegrationOut",
    "IntegrationUpdate",
    "ListResponseApiTokenCensoredOut",
    "ListResponseApplicationOut",
    "ListResponseBackgroundTaskOut",
    "ListResponseEndpointMessageOut",
    "ListResponseEndpointOut",
    "ListResponseEventTypeOut",
    "ListResponseIngestEndpointOut",
    "ListResponseIngestSourceOut",
    "ListResponseIntegrationOut",
    "ListResponseMessageAttemptOut",
    "ListResponseMessageEndpointOut",
    "ListResponseMessageOut",
    "ListResponseOperationalWebhookEndpointOut",
    "MessageAttemptExhaustedEvent",
    "MessageAttemptExhaustedEventData",
    "MessageAttemptFailedData",
    "MessageAttemptFailingEvent",
    "MessageAttemptFailingEventData",
    "MessageAttemptOut",
    "MessageAttemptRecoveredEvent",
    "MessageAttemptRecoveredEventData",
    "MessageAttemptTriggerType",
    "MessageEndpointOut",
    "MessageIn",
    "MessageOut",
    "MessageStatus",
    "OperationalWebhookEndpointHeadersIn",
    "OperationalWebhookEndpointHeadersOut",
    "OperationalWebhookEndpointIn",
    "OperationalWebhookEndpointOut",
    "OperationalWebhookEndpointSecretIn",
    "OperationalWebhookEndpointSecretOut",
    "OperationalWebhookEndpointUpdate",
    "Ordering",
    "PollingEndpointConsumerSeekIn",
    "PollingEndpointConsumerSeekOut",
    "PollingEndpointMessageOut",
    "PollingEndpointOut",
    "RecoverIn",
    "RecoverOut",
    "ReplayIn",
    "ReplayOut",
    "RotateTokenOut",
    "SegmentConfig",
    "SegmentConfigOut",
    "ShopifyConfig",
    "ShopifyConfigOut",
    "SlackConfig",
    "SlackConfigOut",
    "StatusCodeClass",
    "StripeConfig",
    "StripeConfigOut",
    "SvixConfig",
    "SvixConfigOut",
    "ZoomConfig",
    "ZoomConfigOut",
]



# this file is @generated

from .common import BaseModel
from .message_attempt_exhausted_event_data import MessageAttemptExhaustedEventData


class MessageAttemptExhaustedEvent(BaseModel):
    """Sent when a message delivery has failed (all of the retry attempts have been exhausted)."""

    data: MessageAttemptExhaustedEventData

    type: str



# this file is @generated
from enum import Enum


class EndpointDisabledTrigger(str, Enum):
    MANUAL = "manual"
    AUTOMATIC = "automatic"

    def __str__(self) -> str:
        return str(self.value)



import itertools
import uuid
from typing import Any, Dict, List, Optional

import pytest
from pytest_httpserver import HTTPServer
from werkzeug import Request, Response

from svix.api import (
    ApplicationIn,
    ApplicationOut,
    EndpointIn,
    EndpointOut,
    EventTypeIn,
    EventTypeOut,
    MessageIn,
    Svix,
)
from svix.webhooks import Webhook


def _gen_uuid(name: str) -> str:
    return str(uuid.uuid5(uuid.NAMESPACE_DNS, name))


@pytest.fixture
def svix_app_name() -> str:
    return "svix_python_tests"


@pytest.fixture
def event_type_schema() -> Dict[str, Any]:
    return {
        "type": "object",
        "title": "event.test",
        "description": "A dummy event type",
        "properties": {
            "value": {
                "type": "string",
                "description": "A simple string value",
            }
        },
        "required": ["value"],
    }


@pytest.fixture
def endpoint_url() -> str:
    return "http://localhost/webhook/receiver"


def create_svix_app(
    svix_api: Svix, svix_app_name: str, svix_app_uid: str
) -> ApplicationOut:
    return svix_api.application.get_or_create(
        ApplicationIn(name=svix_app_name, uid=svix_app_uid)
    )


def create_svix_event_type(
    svix_api: Svix, event_type_schema: Dict[str, Any]
) -> EventTypeOut:
    return svix_api.event_type.create(
        EventTypeIn(
            name=event_type_schema["title"],
            description=event_type_schema["description"],
            schemas={"1": event_type_schema},
        )
    )


def create_svix_endpoint(
    svix_api: Svix,
    app_uid: str,
    event_type_name: str,
    endpoint_url: str,
    endpoint_uid: str,
    channel: Optional[str] = None,
    metadata: Optional[Dict[str, Any]] = None,
    secret: Optional[str] = None,
) -> EndpointOut:
    return svix_api.endpoint.create(
        app_uid,
        EndpointIn(
            url=endpoint_url,
            uid=endpoint_uid,
            version=1,
            filter_types=[event_type_name],
            channels=[channel] if channel else None,
            metadata=metadata,
            secret=secret,
        ),
    )


def test_svix_application_create(svix_api: Svix, svix_app_name: str) -> None:
    svix_app_uid = _gen_uuid(svix_app_name)
    app = create_svix_app(svix_api, svix_app_name, svix_app_uid)
    assert app.name == svix_app_name
    assert app.uid == svix_app_uid


def test_svix_event_type_create(
    svix_api: Svix, event_type_schema: Dict[str, Any]
) -> None:
    event_type = create_svix_event_type(svix_api, event_type_schema)
    assert event_type.name == event_type_schema["title"]
    assert event_type.description == event_type_schema["description"]
    assert event_type.schemas == {"1": event_type_schema}


def svix_endpoint_create_test_params_ids() -> List[str]:
    ids = []
    for params in itertools.product([False, True], repeat=3):
        with_channel, with_metadata, with_secret = params
        ids.append(
            "/".join(
                [
                    ("with" if with_channel else "without") + " channel",
                    ("with" if with_metadata else "without") + " metadata",
                    ("with" if with_secret else "without") + " secret",
                ]
            )
        )
    return ids


@pytest.mark.parametrize(
    "with_channel,with_metadata,with_secret",
    list(itertools.product([False, True], repeat=3)),
    ids=svix_endpoint_create_test_params_ids(),
)
def test_svix_endpoint_create(
    svix_api: Svix,
    svix_app_name: str,
    event_type_schema: Dict[str, Any],
    endpoint_url: str,
    with_channel: bool,
    with_metadata: bool,
    with_secret: bool,
) -> None:
    svix_app_uid = _gen_uuid(svix_app_name)
    app = create_svix_app(svix_api, svix_app_name, svix_app_uid)
    event_type = create_svix_event_type(svix_api, event_type_schema)
    endpoint_uid = _gen_uuid(endpoint_url)
    channel = "test" if with_channel else None
    metadata = {"test": "test"} if with_metadata else None
    secret = "whsec_" + "e" * 32 if with_secret else None
    assert app.uid
    endpoint = create_svix_endpoint(
        svix_api,
        app.uid,
        event_type.name,
        endpoint_url,
        endpoint_uid,
        channel,
        metadata,
        secret,
    )
    assert endpoint.url == endpoint_url
    assert endpoint.uid == endpoint_uid
    assert endpoint.filter_types == [event_type.name]
    if with_channel:
        assert endpoint.channels == [channel]
    if with_metadata:
        assert endpoint.metadata == metadata
    if with_secret:
        assert svix_api.endpoint.get_secret(app.uid, endpoint_uid).key == secret


@pytest.mark.parametrize(
    "with_channel", [False, True], ids=["without channel", "with channel"]
)
def test_svix_message_create(
    svix_api: Svix,
    svix_app_name: str,
    event_type_schema: Dict[str, Any],
    httpserver: HTTPServer,
    with_channel: bool,
) -> None:
    svix_app_uid = _gen_uuid(svix_app_name)
    create_svix_app(svix_api, svix_app_name, svix_app_uid)
    event_type = create_svix_event_type(svix_api, event_type_schema)

    channel = "test" if with_channel else None
    endpoint_path = "/webhook/receiver/"
    endpoint_url = httpserver.url_for(endpoint_path)
    endpoint_uid = _gen_uuid(endpoint_url)
    create_svix_endpoint(
        svix_api,
        svix_app_uid,
        event_type.name,
        endpoint_url,
        endpoint_uid,
        channel=channel,
    )
    secret = svix_api.endpoint.get_secret(svix_app_uid, endpoint_uid).key

    payload = {"value": "test"}

    def webhook_handler(request: Request) -> Response:
        assert "Svix-Id" in request.headers
        assert "Svix-Timestamp" in request.headers
        assert "Svix-Signature" in request.headers

        webhook = Webhook(secret)
        received_payload = webhook.verify(request.data, request.headers)
        assert received_payload == payload

        return Response("OK")

    httpserver.expect_oneshot_request(
        endpoint_path,
        method="POST",
        json=payload,
    ).respond_with_handler(webhook_handler)

    # send message and check it is received by local http server
    with httpserver.wait() as waiting:
        message_out = svix_api.message.create(
            svix_app_uid,
            MessageIn(
                event_type=event_type.name,
                payload=payload,
                channels=[channel] if channel else None,
            ),
        )

    assert waiting.result

    httpserver.check()  # type: ignore[no-untyped-call]

    assert message_out.event_type == event_type.name
    assert message_out.event_type == event_type.name
    if with_channel:
        assert message_out.channels == [channel]



# this file is @generated
import typing as t

from .common import BaseModel
from .integration_out import IntegrationOut


class ListResponseIntegrationOut(BaseModel):
    data: t.List[IntegrationOut]

    done: bool

    iterator: t.Optional[str]

    prev_iterator: t.Optional[str] = None



# this file is @generated
import typing as t

from .common import BaseModel


class EventExampleIn(BaseModel):
    event_type: str
    """The event type's name"""

    example_index: t.Optional[int] = None
    """If the event type schema contains an array of examples, chooses which one to send.

    Defaults to the first example. Ignored if the schema doesn't contain an array of examples."""



# this file is @generated

from .common import BaseModel
from .message_attempt_failing_event_data import MessageAttemptFailingEventData


class MessageAttemptFailingEvent(BaseModel):
    """Sent after a message has been failing for a few times.
    It's sent on the fourth failure. It complements `message.attempt.exhausted` which is sent after the last failure."""

    data: MessageAttemptFailingEventData

    type: str



# This file is @generated
import typing as t
from dataclasses import dataclass

from ..models import EnvironmentIn, EnvironmentOut
from .common import ApiBase, BaseOptions, serialize_params


@dataclass
class EnvironmentExportOptions(BaseOptions):
    idempotency_key: t.Optional[str] = None

    def _header_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "idempotency-key": self.idempotency_key,
            }
        )


@dataclass
class EnvironmentImportOptions(BaseOptions):
    idempotency_key: t.Optional[str] = None

    def _header_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "idempotency-key": self.idempotency_key,
            }
        )


class EnvironmentAsync(ApiBase):
    async def export(
        self, options: EnvironmentExportOptions = EnvironmentExportOptions()
    ) -> EnvironmentOut:
        """Download a JSON file containing all org-settings and event types."""
        response = await self._request_asyncio(
            method="post",
            path="/api/v1/environment/export",
            path_params={},
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return EnvironmentOut.model_validate(response.json())

    async def import_(
        self,
        environment_in: EnvironmentIn,
        options: EnvironmentImportOptions = EnvironmentImportOptions(),
    ) -> None:
        """Import a configuration into the active organization.

        It doesn't delete anything, only adds / updates what was passed to it."""
        await self._request_asyncio(
            method="post",
            path="/api/v1/environment/import",
            path_params={},
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=environment_in.model_dump_json(exclude_unset=True, by_alias=True),
        )


class Environment(ApiBase):
    def export(
        self, options: EnvironmentExportOptions = EnvironmentExportOptions()
    ) -> EnvironmentOut:
        """Download a JSON file containing all org-settings and event types."""
        response = self._request_sync(
            method="post",
            path="/api/v1/environment/export",
            path_params={},
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return EnvironmentOut.model_validate(response.json())

    def import_(
        self,
        environment_in: EnvironmentIn,
        options: EnvironmentImportOptions = EnvironmentImportOptions(),
    ) -> None:
        """Import a configuration into the active organization.

        It doesn't delete anything, only adds / updates what was passed to it."""
        self._request_sync(
            method="post",
            path="/api/v1/environment/import",
            path_params={},
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=environment_in.model_dump_json(exclude_unset=True, by_alias=True),
        )



# this file is @generated
import typing as t

from .common import BaseModel


class OperationalWebhookEndpointIn(BaseModel):
    description: t.Optional[str] = None

    disabled: t.Optional[bool] = None

    filter_types: t.Optional[t.List[str]] = None

    metadata: t.Optional[t.Dict[str, str]] = None

    rate_limit: t.Optional[int] = None

    secret: t.Optional[str] = None
    """The endpoint's verification secret.

    Format: `base64` encoded random bytes optionally prefixed with `whsec_`.
    It is recommended to not set this and let the server generate the secret."""

    uid: t.Optional[str] = None
    """Optional unique identifier for the endpoint."""

    url: str



# This file is @generated
import typing as t
from dataclasses import dataclass

from deprecated import deprecated

from .. import models
from ..models import (
    IntegrationIn,
    IntegrationKeyOut,
    IntegrationOut,
    IntegrationUpdate,
    ListResponseIntegrationOut,
)
from .common import ApiBase, BaseOptions, serialize_params


@dataclass
class IntegrationListOptions(BaseOptions):
    limit: t.Optional[int] = None
    """Limit the number of returned items"""
    iterator: t.Optional[str] = None
    """The iterator returned from a prior invocation"""
    order: t.Optional[models.Ordering] = None
    """The sorting order of the returned items"""

    def _query_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "limit": self.limit,
                "iterator": self.iterator,
                "order": self.order,
            }
        )


@dataclass
class IntegrationCreateOptions(BaseOptions):
    idempotency_key: t.Optional[str] = None

    def _header_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "idempotency-key": self.idempotency_key,
            }
        )


@dataclass
class IntegrationRotateKeyOptions(BaseOptions):
    idempotency_key: t.Optional[str] = None

    def _header_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "idempotency-key": self.idempotency_key,
            }
        )


class IntegrationAsync(ApiBase):
    async def list(
        self, app_id: str, options: IntegrationListOptions = IntegrationListOptions()
    ) -> ListResponseIntegrationOut:
        """List the application's integrations."""
        response = await self._request_asyncio(
            method="get",
            path="/api/v1/app/{app_id}/integration",
            path_params={
                "app_id": app_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return ListResponseIntegrationOut.model_validate(response.json())

    async def create(
        self,
        app_id: str,
        integration_in: IntegrationIn,
        options: IntegrationCreateOptions = IntegrationCreateOptions(),
    ) -> IntegrationOut:
        """Create an integration."""
        response = await self._request_asyncio(
            method="post",
            path="/api/v1/app/{app_id}/integration",
            path_params={
                "app_id": app_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=integration_in.model_dump_json(exclude_unset=True, by_alias=True),
        )
        return IntegrationOut.model_validate(response.json())

    async def get(self, app_id: str, integ_id: str) -> IntegrationOut:
        """Get an integration."""
        response = await self._request_asyncio(
            method="get",
            path="/api/v1/app/{app_id}/integration/{integ_id}",
            path_params={
                "app_id": app_id,
                "integ_id": integ_id,
            },
        )
        return IntegrationOut.model_validate(response.json())

    async def update(
        self, app_id: str, integ_id: str, integration_update: IntegrationUpdate
    ) -> IntegrationOut:
        """Update an integration."""
        response = await self._request_asyncio(
            method="put",
            path="/api/v1/app/{app_id}/integration/{integ_id}",
            path_params={
                "app_id": app_id,
                "integ_id": integ_id,
            },
            json_body=integration_update.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return IntegrationOut.model_validate(response.json())

    async def delete(self, app_id: str, integ_id: str) -> None:
        """Delete an integration."""
        await self._request_asyncio(
            method="delete",
            path="/api/v1/app/{app_id}/integration/{integ_id}",
            path_params={
                "app_id": app_id,
                "integ_id": integ_id,
            },
        )

    @deprecated
    async def get_key(self, app_id: str, integ_id: str) -> IntegrationKeyOut:
        """Get an integration's key."""
        response = await self._request_asyncio(
            method="get",
            path="/api/v1/app/{app_id}/integration/{integ_id}/key",
            path_params={
                "app_id": app_id,
                "integ_id": integ_id,
            },
        )
        return IntegrationKeyOut.model_validate(response.json())

    async def rotate_key(
        self,
        app_id: str,
        integ_id: str,
        options: IntegrationRotateKeyOptions = IntegrationRotateKeyOptions(),
    ) -> IntegrationKeyOut:
        """Rotate the integration's key. The previous key will be immediately revoked."""
        response = await self._request_asyncio(
            method="post",
            path="/api/v1/app/{app_id}/integration/{integ_id}/key/rotate",
            path_params={
                "app_id": app_id,
                "integ_id": integ_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return IntegrationKeyOut.model_validate(response.json())


class Integration(ApiBase):
    def list(
        self, app_id: str, options: IntegrationListOptions = IntegrationListOptions()
    ) -> ListResponseIntegrationOut:
        """List the application's integrations."""
        response = self._request_sync(
            method="get",
            path="/api/v1/app/{app_id}/integration",
            path_params={
                "app_id": app_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return ListResponseIntegrationOut.model_validate(response.json())

    def create(
        self,
        app_id: str,
        integration_in: IntegrationIn,
        options: IntegrationCreateOptions = IntegrationCreateOptions(),
    ) -> IntegrationOut:
        """Create an integration."""
        response = self._request_sync(
            method="post",
            path="/api/v1/app/{app_id}/integration",
            path_params={
                "app_id": app_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=integration_in.model_dump_json(exclude_unset=True, by_alias=True),
        )
        return IntegrationOut.model_validate(response.json())

    def get(self, app_id: str, integ_id: str) -> IntegrationOut:
        """Get an integration."""
        response = self._request_sync(
            method="get",
            path="/api/v1/app/{app_id}/integration/{integ_id}",
            path_params={
                "app_id": app_id,
                "integ_id": integ_id,
            },
        )
        return IntegrationOut.model_validate(response.json())

    def update(
        self, app_id: str, integ_id: str, integration_update: IntegrationUpdate
    ) -> IntegrationOut:
        """Update an integration."""
        response = self._request_sync(
            method="put",
            path="/api/v1/app/{app_id}/integration/{integ_id}",
            path_params={
                "app_id": app_id,
                "integ_id": integ_id,
            },
            json_body=integration_update.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return IntegrationOut.model_validate(response.json())

    def delete(self, app_id: str, integ_id: str) -> None:
        """Delete an integration."""
        self._request_sync(
            method="delete",
            path="/api/v1/app/{app_id}/integration/{integ_id}",
            path_params={
                "app_id": app_id,
                "integ_id": integ_id,
            },
        )

    @deprecated
    def get_key(self, app_id: str, integ_id: str) -> IntegrationKeyOut:
        """Get an integration's key."""
        response = self._request_sync(
            method="get",
            path="/api/v1/app/{app_id}/integration/{integ_id}/key",
            path_params={
                "app_id": app_id,
                "integ_id": integ_id,
            },
        )
        return IntegrationKeyOut.model_validate(response.json())

    def rotate_key(
        self,
        app_id: str,
        integ_id: str,
        options: IntegrationRotateKeyOptions = IntegrationRotateKeyOptions(),
    ) -> IntegrationKeyOut:
        """Rotate the integration's key. The previous key will be immediately revoked."""
        response = self._request_sync(
            method="post",
            path="/api/v1/app/{app_id}/integration/{integ_id}/key/rotate",
            path_params={
                "app_id": app_id,
                "integ_id": integ_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return IntegrationKeyOut.model_validate(response.json())



# this file is @generated

from .common import BaseModel
from .endpoint_deleted_event_data import EndpointDeletedEventData


class EndpointDeletedEvent(BaseModel):
    """Sent when an endpoint is deleted."""

    data: EndpointDeletedEventData

    type: str



# this file is @generated
import typing as t

from .api_token_censored_out import ApiTokenCensoredOut
from .common import BaseModel


class ListResponseApiTokenCensoredOut(BaseModel):
    data: t.List[ApiTokenCensoredOut]

    done: bool

    iterator: t.Optional[str]

    prev_iterator: t.Optional[str] = None



# this file is @generated
import typing as t

from .common import BaseModel
from .message_attempt_failed_data import MessageAttemptFailedData


class MessageAttemptExhaustedEventData(BaseModel):
    """Sent when a message delivery has failed (all of the retry attempts have been exhausted) as a "message.attempt.exhausted" type or after it's failed four times as a "message.attempt.failing" event."""

    app_id: str
    """The Application's ID."""

    app_uid: t.Optional[str] = None
    """The Application's UID."""

    endpoint_id: str
    """The Endpoint's ID."""

    last_attempt: MessageAttemptFailedData

    msg_event_id: t.Optional[str] = None
    """The Message's UID."""

    msg_id: str
    """The Message's ID."""



# this file is @generated
import typing as t

from .common import BaseModel
from .ingest_endpoint_out import IngestEndpointOut


class ListResponseIngestEndpointOut(BaseModel):
    data: t.List[IngestEndpointOut]

    done: bool

    iterator: t.Optional[str]

    prev_iterator: t.Optional[str] = None



# this file is @generated
import typing as t

from pydantic import ModelWrapValidatorHandler, model_validator
from typing_extensions import Self

from .adobe_sign_config import AdobeSignConfig
from .common import BaseModel
from .cron_config import CronConfig
from .docusign_config import DocusignConfig
from .github_config import GithubConfig
from .hubspot_config import HubspotConfig
from .segment_config import SegmentConfig
from .shopify_config import ShopifyConfig
from .slack_config import SlackConfig
from .stripe_config import StripeConfig
from .svix_config import SvixConfig
from .zoom_config import ZoomConfig


class IngestSourceIn(BaseModel):
    name: str

    uid: t.Optional[str] = None
    """The Source's UID."""

    type: t.Union[
        t.Literal["generic-webhook"],
        t.Literal["cron"],
        t.Literal["adobe-sign"],
        t.Literal["beehiiv"],
        t.Literal["brex"],
        t.Literal["clerk"],
        t.Literal["docusign"],
        t.Literal["github"],
        t.Literal["guesty"],
        t.Literal["hubspot"],
        t.Literal["incident-io"],
        t.Literal["lithic"],
        t.Literal["nash"],
        t.Literal["pleo"],
        t.Literal["replicate"],
        t.Literal["resend"],
        t.Literal["safebase"],
        t.Literal["sardine"],
        t.Literal["segment"],
        t.Literal["shopify"],
        t.Literal["slack"],
        t.Literal["stripe"],
        t.Literal["stych"],
        t.Literal["svix"],
        t.Literal["zoom"],
    ]
    config: t.Union[
        t.Dict[str, t.Any],
        CronConfig,
        AdobeSignConfig,
        SvixConfig,
        DocusignConfig,
        GithubConfig,
        HubspotConfig,
        SegmentConfig,
        ShopifyConfig,
        SlackConfig,
        StripeConfig,
        ZoomConfig,
    ]

    @model_validator(mode="wrap")
    @classmethod
    def validate_model(
        cls, data: t.Any, handler: ModelWrapValidatorHandler[Self]
    ) -> Self:
        output = handler(data)
        if output.type == "generic-webhook":
            output.config = data.get("config", {})
        elif output.type == "cron":
            output.config = CronConfig.model_validate(data.get("config", {}))
        elif output.type == "adobe-sign":
            output.config = AdobeSignConfig.model_validate(data.get("config", {}))
        elif output.type == "beehiiv":
            output.config = SvixConfig.model_validate(data.get("config", {}))
        elif output.type == "brex":
            output.config = SvixConfig.model_validate(data.get("config", {}))
        elif output.type == "clerk":
            output.config = SvixConfig.model_validate(data.get("config", {}))
        elif output.type == "docusign":
            output.config = DocusignConfig.model_validate(data.get("config", {}))
        elif output.type == "github":
            output.config = GithubConfig.model_validate(data.get("config", {}))
        elif output.type == "guesty":
            output.config = SvixConfig.model_validate(data.get("config", {}))
        elif output.type == "hubspot":
            output.config = HubspotConfig.model_validate(data.get("config", {}))
        elif output.type == "incident-io":
            output.config = SvixConfig.model_validate(data.get("config", {}))
        elif output.type == "lithic":
            output.config = SvixConfig.model_validate(data.get("config", {}))
        elif output.type == "nash":
            output.config = SvixConfig.model_validate(data.get("config", {}))
        elif output.type == "pleo":
            output.config = SvixConfig.model_validate(data.get("config", {}))
        elif output.type == "replicate":
            output.config = SvixConfig.model_validate(data.get("config", {}))
        elif output.type == "resend":
            output.config = SvixConfig.model_validate(data.get("config", {}))
        elif output.type == "safebase":
            output.config = SvixConfig.model_validate(data.get("config", {}))
        elif output.type == "sardine":
            output.config = SvixConfig.model_validate(data.get("config", {}))
        elif output.type == "segment":
            output.config = SegmentConfig.model_validate(data.get("config", {}))
        elif output.type == "shopify":
            output.config = ShopifyConfig.model_validate(data.get("config", {}))
        elif output.type == "slack":
            output.config = SlackConfig.model_validate(data.get("config", {}))
        elif output.type == "stripe":
            output.config = StripeConfig.model_validate(data.get("config", {}))
        elif output.type == "stych":
            output.config = SvixConfig.model_validate(data.get("config", {}))
        elif output.type == "svix":
            output.config = SvixConfig.model_validate(data.get("config", {}))
        elif output.type == "zoom":
            output.config = ZoomConfig.model_validate(data.get("config", {}))
        else:
            raise ValueError(f"Unexpected type `{output.type}`")
        return output



# this file is @generated
import typing as t
from datetime import datetime

from .common import BaseModel
from .connector_out import ConnectorOut
from .event_type_out import EventTypeOut


class EnvironmentOut(BaseModel):
    created_at: datetime

    event_types: t.List[EventTypeOut]

    settings: t.Optional[t.Dict[str, t.Any]]

    transformation_templates: t.List[ConnectorOut]

    version: t.Optional[int] = None
